@@ -1,6 +1,6 @@
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -58,7 +58,7 @@ from Cryptodome.Cipher import AES, DES3, ARC4, DES
 from Cryptodome.Hash import HMAC, MD4, MD5, SHA
 from Cryptodome.Protocol.KDF import PBKDF2
 from Cryptodome.Util.number import GCD as gcd
-from six import b, PY3, indexbytes
+from six import b, PY3, indexbytes, binary_type
 
 
 def get_random_bytes(lenBytes):
@@ -366,6 +366,11 @@ class _DESCBC(_SimplifiedEnctype):
     def string_to_key(cls, string, salt, params):
         if params is not None and params != b'':
             raise ValueError('Invalid DES string-to-key parameters')
+        if not isinstance(string, binary_type):
+            string = string.encode("utf-8")
+        if not isinstance(salt, binary_type):
+            salt = salt.encode("utf-8")
+
         key = cls.mit_des_string_to_key(string, salt)
         return key
     
@@ -408,6 +413,11 @@ class _DES3CBC(_SimplifiedEnctype):
     def string_to_key(cls, string, salt, params):
         if params is not None and params != b'':
             raise ValueError('Invalid DES3 string-to-key parameters')
+        if not isinstance(string, binary_type):
+            string = string.encode("utf-8")
+        if not isinstance(salt, binary_type):
+            salt = salt.encode("utf-8")
+
         k = cls.random_to_key(_nfold(string + salt, 21))
         return cls.derive(k, b'kerberos')
 
@@ -433,6 +443,11 @@ class _AESEnctype(_SimplifiedEnctype):
 
     @classmethod
     def string_to_key(cls, string, salt, params):
+        if not isinstance(string, binary_type):
+            string = string.encode("utf-8")
+        if not isinstance(salt, binary_type):
+            salt = salt.encode("utf-8")
+
         (iterations,) = unpack('>L', params or b'\x00\x00\x10\x00')
         prf = lambda p, s: HMAC.new(p, s, SHA).digest()
         seed = PBKDF2(string, salt, cls.seedsize, iterations, prf)
